/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gconf/gconf-client.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <gst/pbutils/pbutils.h>
#include <gst/pbutils/install-plugins.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <string.h>
#include "config.h"
#include "prefs.h"
#include "trayicon.h"
#include "gui.h"
#include "rec_tech.h"
#include "rb_gst_media_types.h"
#include "rb_missing_plugins.h"

extern GtkWidget *preset_combo;
extern GtkAdjustment *adj;

extern int mom_ps;
extern gnomeradio_settings settings;

extern gboolean main_visible;

static GtkWidget *device_entry, *mixer_entry;
static GtkWidget *mute_on_exit_cb;
static GtkWidget *save_button, *move_up_button, *move_down_button, *add_button, *remove_button;
static GtkWidget *audio_profile_combo;
static GtkWidget *install_plugins_button;
static GtkWidget *list_view;
static GtkListStore *list_store;
static GtkTreeSelection *selection;

gboolean save_settings(void)
{
	gint i, count;
	gchar *buffer;
	preset *ps;
	GConfClient* client = NULL;
	
	if (!gconf_is_initialized())
		return FALSE;

	client = gconf_client_get_default();
	if (!client)
		return FALSE;
	
	/* Store general settings */
	gconf_client_set_string(client, "/apps/gnomeradio/device", settings.device, NULL);
	gconf_client_set_string(client, "/apps/gnomeradio/driver", settings.driver, NULL);
	gconf_client_set_string(client, "/apps/gnomeradio/mixer", settings.mixer, NULL);
	gconf_client_set_bool(client, "/apps/gnomeradio/mute-on-exit", settings.mute_on_exit, NULL);
	/*gconf_client_set_float(client, "/apps/gnomeradio/volume", volume->value, NULL);*/
	gconf_client_set_float(client, "/apps/gnomeradio/last-freq", gtk_adjustment_get_value(adj)/STEPS, NULL);

	/* Store recording settings */
/*	gconf_client_set_string(client, "/apps/gnomeradio/recording/audiodevice", rec_settings.audiodevice, NULL);
	gconf_client_set_bool(client, "/apps/gnomeradio/recording/record-as-mp3", rec_settings.mp3, NULL);
	gconf_client_set_string(client, "/apps/gnomeradio/recording/sample-rate", rec_settings.rate, NULL);
	gconf_client_set_string(client, "/apps/gnomeradio/recording/sample-format", rec_settings.sample, NULL);
	gconf_client_set_bool(client, "/apps/gnomeradio/recording/record-in-stereo", rec_settings.stereo, NULL);
	gconf_client_set_string(client, "/apps/gnomeradio/recording/encoder", rec_settings.encoder, NULL);
	gconf_client_set_string(client, "/apps/gnomeradio/recording/bitrate", rec_settings.bitrate, NULL);
*/

	gconf_client_set_string(client, "/apps/gnomeradio/recording/destination", rec_settings.destination, NULL);
	gconf_client_set_string(client, "/apps/gnomeradio/recording/profile", rec_settings.profile, NULL);

	/* Store the presets */
	count = g_list_length(settings.presets);
	gconf_client_set_int(client, "/apps/gnomeradio/presets/presets", count, NULL);
	for (i = 0; i < count; i++)
	{
		ps = g_list_nth_data(settings.presets, i);
		buffer = g_strdup_printf("/apps/gnomeradio/presets/%d/name", i);
		gconf_client_set_string(client, buffer, ps->title, NULL); 
		g_free(buffer);
		buffer = g_strdup_printf("/apps/gnomeradio/presets/%d/freqency", i);
		gconf_client_set_float(client, buffer, ps->freq, NULL); 
		g_free(buffer);
	}	
	gconf_client_set_int(client, "/apps/gnomeradio/presets/last", mom_ps, NULL);
	/*g_print("Storing Settings in GConf database\n");*/
	
	return TRUE;
}			

gboolean load_settings(void)
{
	gint i, count;
	gchar *buffer;
	preset *ps;
	GConfClient *client = NULL;
	double freq;
	
	settings.presets = NULL;
		
	if (!gconf_is_initialized())
		return FALSE;
	
	client = gconf_client_get_default();
	if (!client)
		return FALSE;

	/* Load general settings */
	settings.device = gconf_client_get_string(client, "/apps/gnomeradio/device" , NULL);
	if (!settings.device)
		settings.device = g_strdup("/dev/radio0");
	settings.driver = gconf_client_get_string(client, "/apps/gnomeradio/driver" , NULL);
	if (!settings.driver)
		settings.driver = g_strdup("any");
	settings.mixer = gconf_client_get_string(client, "/apps/gnomeradio/mixer", NULL);
	if (!settings.mixer)
		settings.mixer = g_strdup("hw:0/Line");
	settings.mute_on_exit = gconf_client_get_bool(client, "/apps/gnomeradio/mute-on-exit", NULL);
	/*volume->value = gconf_client_get_float(client, "/apps/gnomeradio/volume", NULL);*/
	freq = gconf_client_get_float(client, "/apps/gnomeradio/last-freq", NULL);
	if ((freq < FREQ_MIN) || (freq > FREQ_MAX))
		gtk_adjustment_set_value(adj, FREQ_MIN * STEPS);
	else
		gtk_adjustment_set_value(adj, freq * STEPS);
	
	/* Load recording settings */
/*	rec_settings.audiodevice = gconf_client_get_string(client, "/apps/gnomeradio/recording/audiodevice", NULL);
	if (!rec_settings.audiodevice)
		rec_settings.audiodevice = g_strdup("/dev/audio");
	rec_settings.mp3 = gconf_client_get_bool(client, "/apps/gnomeradio/recording/record-as-mp3", NULL);
	rec_settings.rate = gconf_client_get_string(client, "/apps/gnomeradio/recording/sample-rate", NULL);
	if (!rec_settings.rate)
		rec_settings.rate = g_strdup("44100");
	rec_settings.sample = gconf_client_get_string(client, "/apps/gnomeradio/recording/sample-format", NULL);
	if (!rec_settings.sample)
		rec_settings.sample = g_strdup("16");
	rec_settings.stereo = gconf_client_get_bool(client, "/apps/gnomeradio/recording/record-in-stereo", NULL);
	rec_settings.encoder = gconf_client_get_string(client, "/apps/gnomeradio/recording/encoder", NULL);
	if (!rec_settings.encoder)
		rec_settings.encoder = g_strdup("oggenc");
	rec_settings.bitrate = gconf_client_get_string(client, "/apps/gnomeradio/recording/bitrate", NULL);
	if (!rec_settings.bitrate)
		rec_settings.bitrate = g_strdup("192");*/

	rec_settings.destination = gconf_client_get_string(client, "/apps/gnomeradio/recording/destination", NULL);
	if (!rec_settings.destination)
		rec_settings.destination = g_strdup(g_get_home_dir());
	rec_settings.profile = gconf_client_get_string(client, "/apps/gnomeradio/recording/profile", NULL);
	if (!rec_settings.profile)
		rec_settings.profile = g_strdup("audio/x-vorbis");
	
	/* Load the presets */
	count = gconf_client_get_int(client, "/apps/gnomeradio/presets/presets", NULL);
	for (i = 0; i < count; i++)
	{
		ps = malloc(sizeof(preset));
		buffer = g_strdup_printf("/apps/gnomeradio/presets/%d/name", i);
		ps->title = gconf_client_get_string(client, buffer, NULL); 
		g_free(buffer);
		if (!ps->title)
			ps->title = g_strdup(_("unnamed"));
		buffer = g_strdup_printf("/apps/gnomeradio/presets/%d/freqency", i);
		freq = gconf_client_get_float(client, buffer, NULL); 
		if ((freq < FREQ_MIN) || (freq > FREQ_MAX))
			ps->freq = FREQ_MIN;
		else
			ps->freq = freq;
		g_free(buffer);
		settings.presets = g_list_prepend(settings.presets, (gpointer)ps);
	}	
	settings.presets = g_list_reverse(settings.presets);

	mom_ps = gconf_client_get_int(client, "/apps/gnomeradio/presets/last", NULL);
	if (mom_ps >= count)
		mom_ps = -1;

	return TRUE;
}			
	
static void mute_on_exit_toggled_cb(GtkWidget* widget, gpointer data)
{
	settings.mute_on_exit = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mute_on_exit_cb));
}	

static gboolean device_entry_activate_cb(GtkWidget *widget, gpointer data)
{
	const gchar *text;

	text = gtk_entry_get_text(GTK_ENTRY(device_entry));

	if (!strcmp(settings.device, text) || (text[0] == '\0'))
		return FALSE;
	
	if (settings.device)
		g_free(settings.device);

	settings.device = g_strdup(text);
	
	start_radio(TRUE, data);
	
	return FALSE;
}

static gboolean mixer_entry_activate_cb(GtkWidget *widget, gpointer data)
{
	const gchar *text;

	text = gtk_entry_get_text(GTK_ENTRY(mixer_entry));

	if (!strcmp(settings.mixer, text) || (text[0] == '\0'))
		return FALSE;
	
	if (settings.mixer)
		g_free(settings.mixer);

	settings.mixer = g_strdup(text);
	
	start_mixer(TRUE, data);
	
	return FALSE;
}

/*static gboolean bitrate_combo_change_cb(GtkComboBox *combo, gpointer data)
{
	GList *bitrates;
	gint active;
	gchar *bitrate;

	g_assert(combo);
	bitrates = g_object_get_data(G_OBJECT(combo), "bitrates");
	active = gtk_combo_box_get_active(combo);
	g_assert(active > -1);
	
	bitrate = (gchar*)g_list_nth_data(bitrates, active);
	g_assert(bitrate);

	if (rec_settings.bitrate) g_free(rec_settings.bitrate);
	rec_settings.bitrate = g_strdup(bitrate);
	
	return FALSE;
}*/

/*static gboolean encoder_combo_change_cb(GtkComboBox *combo, gpointer bitrate_combo)
{
	GList *encoders;
	gint active;
	gchar *encoder;
	
	g_assert(combo);
	encoders = g_object_get_data(G_OBJECT(combo), "encoders");
	active = gtk_combo_box_get_active(combo);
	g_assert(active > -1);
	
	encoder = (gchar*)g_list_nth_data(encoders, active);
	g_assert(encoder);

	if (g_str_equal(encoder, _("Wave file"))) rec_settings.mp3 = FALSE;
	else {
		rec_settings.mp3 = TRUE;
		if (rec_settings.encoder) g_free(rec_settings.encoder);
		rec_settings.encoder = g_strdup(encoder);
	}
	gtk_widget_set_sensitive(bitrate_combo, rec_settings.mp3);
	
	return FALSE;
}*/

static void audio_profile_combo_change_cb(GtkWidget *widget, gpointer user_data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	char *media_type = NULL;
	GstEncodingProfile *profile;

	model = gtk_combo_box_get_model(GTK_COMBO_BOX(widget));
	/* get selected media type */
	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter) == FALSE)
		return;
	gtk_tree_model_get (GTK_TREE_MODEL (model),
			    &iter,
			    0, &media_type,
			    2, &profile,
			    -1);

	rec_settings.profile = g_strdup(media_type);

	/* indicate whether additional plugins are required to encode in this format */
	if (rb_gst_check_missing_plugins (profile, NULL, NULL)) {
		gtk_widget_set_visible (install_plugins_button, TRUE);

		gtk_widget_set_sensitive (install_plugins_button,
					gst_install_plugins_supported ());
	} else {
		gtk_widget_set_visible (install_plugins_button, FALSE);
	}
	g_free (media_type);
}

static void
audio_profile_chooser_set_active(GtkWidget *widget, const char *profile)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	gboolean done;

	done = FALSE;
	model = gtk_combo_box_get_model(GTK_COMBO_BOX(widget));
	if (gtk_tree_model_get_iter_first(model, &iter)) {
		do {
			char *media_type;

			gtk_tree_model_get(model, &iter, 0, &media_type, -1);
			if (g_strcmp0(media_type, profile) == 0) {
				gtk_combo_box_set_active_iter(GTK_COMBO_BOX(widget), &iter);
				done = TRUE;
			}
			g_free (media_type);
		} while (done == FALSE && gtk_tree_model_iter_next(model, &iter));
	}

	if (done == FALSE) {
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(widget), NULL);
	}
}

static GtkWidget *audio_profile_choose_new(void)
{
	GstEncodingTarget *target;
	const GList *p;
	GtkWidget *combo;
	GtkCellRenderer *renderer;
	GtkTreeModel *model;

	model = GTK_TREE_MODEL(gtk_tree_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER));

	target = rb_gst_get_default_encoding_target();
	for (p = gst_encoding_target_get_profiles(target); p != NULL; p = p->next) {
		GstEncodingProfile *profile = GST_ENCODING_PROFILE (p->data);
		char *media_type;

		media_type = rb_gst_encoding_profile_get_media_type(profile);
		if (media_type == NULL) {
			continue;
		}
		gtk_tree_store_insert_with_values(GTK_TREE_STORE(model),
			          		  NULL, NULL, -1,
			          		  0, media_type,
			          		  1, gst_encoding_profile_get_description(profile),
			          		  2, profile, -1);
		g_free(media_type);
	}

	combo = gtk_combo_box_new_with_model(model);
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo), renderer, "text", 1, NULL);

	return GTK_WIDGET(combo);
}

static void
audio_plugin_install_done_cb (gpointer inst, gboolean retry, gpointer user_data)
{
	audio_profile_combo_change_cb (audio_profile_combo, user_data);
}

static void
audio_profile_install_plugins_cb (GtkWidget *widget, gpointer user_data)
{
	GstEncodingProfile *profile;
	char **details;
	GClosure *closure;

	/* get profile */
	profile = rb_gst_get_encoding_profile (rec_settings.profile);
	if (profile == NULL) {
		g_free (rec_settings.profile);
		return;
	}
	g_free (rec_settings.profile);

	/* get plugin details */
	if (rb_gst_check_missing_plugins (profile, &details, NULL) == FALSE) {
		return;
	}

	/* attempt installation */
	closure = g_cclosure_new ((GCallback) audio_plugin_install_done_cb,
				  g_object_ref (audio_profile_combo),
				  (GClosureNotify) g_object_unref);
	g_closure_set_marshal (closure, g_cclosure_marshal_VOID__BOOLEAN);

	rb_missing_plugins_install ((const char **)details, TRUE, closure);

	g_closure_sink (closure);
	g_strfreev (details);
}

static void add_button_clicked_cb(GtkWidget *widget, gpointer data)
{
	preset *ps;
	gchar *buffer;
	GtkTreeIter iter = {0};
	GtkAdjustment* v_scb;
	GtkTreePath *path = NULL;
	GtkWidget *menuitem;
	
	ps = malloc(sizeof(preset));
	ps->title = g_strdup(_("unnamed"));
	ps->freq = rint(gtk_adjustment_get_value(adj)) / STEPS;
	settings.presets = g_list_append(settings.presets, (gpointer) ps);
	buffer = g_strdup_printf("%.2f", ps->freq);

	gtk_list_store_append(list_store, &iter);
	gtk_list_store_set(GTK_LIST_STORE(list_store), &iter, 0, ps->title, 1, buffer, -1);

	g_free(buffer);
	gtk_tree_selection_unselect_all(selection);
	
	v_scb = gtk_scrollable_get_vadjustment(GTK_SCROLLABLE(list_view));
	gtk_adjustment_set_value(v_scb, gtk_adjustment_get_upper(v_scb));
	
	if (main_visible) {
		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(preset_combo), ps->title);
		mom_ps = g_list_length(settings.presets) - 1;
		preset_combo_set_item(mom_ps);

		menuitem = gtk_menu_item_new_with_label(ps->title); 
			
		gtk_menu_shell_insert(GTK_MENU_SHELL(tray_menu), menuitem, mom_ps);		
		g_signal_connect(G_OBJECT(menuitem), "activate", (GCallback)preset_menuitem_activate_cb, GINT_TO_POINTER (mom_ps));
		gtk_widget_show(menuitem);
	}

	buffer = g_strdup_printf("%d", g_list_length(settings.presets) - 1);
	path = gtk_tree_path_new_from_string(buffer);
	g_free(buffer);
	gtk_tree_view_set_cursor(GTK_TREE_VIEW(list_view), path, NULL, FALSE);
	gtk_tree_path_free(path);

	gtk_widget_set_sensitive(save_button, TRUE);
}

static void remove_button_clicked_cb(GtkWidget *widget, gpointer data)
{
	GtkTreePath *path = NULL;
	GtkTreeViewColumn *focus_column = NULL;
	GtkTreeIter iter;
	preset *ps;
	int *row;
	GList* menuitems;
	GtkWidget *menuitem;
	
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(list_view), &path, &focus_column);
	
	if (!path) return;

	row = gtk_tree_path_get_indices(path);
	g_assert(row);
	g_assert(*row < g_list_length(settings.presets));

	ps = g_list_nth_data(settings.presets, *row);
	g_assert(ps);	
	settings.presets = g_list_remove(settings.presets, (gpointer)ps);
	g_free(ps->title);
	g_free(ps);
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(list_store), &iter, path);
	gtk_list_store_remove(list_store, &iter);

	if (main_visible) {
		gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(preset_combo), *row + 1);
		if (--mom_ps < 0) mom_ps = 0;
		if (settings.presets == NULL) mom_ps = -1;
		preset_combo_set_item(mom_ps);

		menuitems = gtk_container_get_children(GTK_CONTAINER(tray_menu));
		g_assert(*row < g_list_length(menuitems));
		menuitem = g_list_nth_data(menuitems, *row);
		gtk_widget_destroy(menuitem);
	}
	
	gtk_tree_path_prev(path);
	gtk_tree_view_set_cursor(GTK_TREE_VIEW(list_view), path, NULL, FALSE);
	gtk_tree_path_free(path);

	if (settings.presets == NULL) {
 		gtk_widget_set_sensitive(save_button, FALSE);
	} else {
		gtk_widget_set_sensitive(save_button, TRUE);
	}
}

static void move_up_button_clicked_cb(GtkWidget * widget, gpointer data)
{
	GtkTreeIter iter, iter2;
	GtkTreePath *path, *path2;
	gint pos;
	preset *ps;
	int *row;
	GList* menuitems;
	GtkWidget *menuitem;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list_view));
	gtk_tree_selection_get_selected(selection, NULL, &iter);
	path = gtk_tree_model_get_path(GTK_TREE_MODEL(list_store), &iter);

	path2 = path;
	gtk_tree_path_prev(path2);

	if (gtk_tree_model_get_iter(GTK_TREE_MODEL(list_store), &iter2, path2)) {
		// we have a previous entry...
		gtk_list_store_swap(GTK_LIST_STORE(GTK_TREE_MODEL(list_store)), &iter, &iter2);
	}

	row = gtk_tree_path_get_indices(path);
	g_assert(row);
	g_assert(*row < g_list_length(settings.presets));

	ps = g_list_nth_data(settings.presets, *row);
	g_assert(ps);
	pos = g_list_index(settings.presets, (gpointer)ps);
	pos++;

	settings.presets = g_list_remove(settings.presets, (gpointer)ps);
	settings.presets = g_list_insert(settings.presets, (gpointer)ps, pos);

	if (main_visible) {
		gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(preset_combo), *row + 1);
		gtk_combo_box_text_insert_text(GTK_COMBO_BOX_TEXT(preset_combo), *row + 2, ps->title);
		mom_ps = *row;
		preset_combo_set_item(mom_ps);

		menuitems = gtk_container_get_children(GTK_CONTAINER(tray_menu));
		g_assert(mom_ps < g_list_length(menuitems));
		menuitem = g_list_nth_data(menuitems, mom_ps);
		gtk_widget_destroy(menuitem);
		menuitem = gtk_menu_item_new_with_label(ps->title);

		gtk_menu_shell_insert(GTK_MENU_SHELL(tray_menu), menuitem, *row + 1);
		g_signal_connect(G_OBJECT(menuitem), "activate", (GCallback)preset_menuitem_activate_cb, GINT_TO_POINTER (mom_ps));
		gtk_widget_show(menuitem);
	}

	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list_view), path, NULL, FALSE, 0, 0);
	gtk_tree_path_free(path);

	if (pos - 1 <= 0) {
		gtk_widget_set_sensitive(move_up_button, FALSE);
	} else {
		gtk_widget_set_sensitive(move_up_button, TRUE);
	}
	gtk_widget_set_sensitive(move_down_button, TRUE);
}

static void move_down_button_clicked_cb(GtkWidget * widget, gpointer data)
{
	GtkTreeIter iter, iter2;
	GtkTreePath *path, *path2;
	gint pos, count;
	preset *ps;
	int *row;
	GList* menuitems;
	GtkWidget *menuitem;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list_view));
	gtk_tree_selection_get_selected(selection, NULL, &iter);
	path = gtk_tree_model_get_path(GTK_TREE_MODEL(list_store), &iter);

	path2 = path;
	gtk_tree_path_next(path2);

	if (gtk_tree_model_get_iter(GTK_TREE_MODEL(list_store), &iter2, path2)) {
		// we have a next entry...
		gtk_list_store_swap(GTK_LIST_STORE(GTK_TREE_MODEL(list_store)), &iter, &iter2);
	}

	row = gtk_tree_path_get_indices(path);
	g_assert(row);
	g_assert(*row < g_list_length(settings.presets));

	ps = g_list_nth_data(settings.presets, *row);
	g_assert(ps);

	count = g_list_length(settings.presets);
	pos = g_list_index(settings.presets, (gpointer)ps);
	pos--;
	if (pos == count)
		pos = 0;

	settings.presets = g_list_remove(settings.presets, (gpointer)ps);
	settings.presets = g_list_insert(settings.presets, (gpointer)ps, pos);

	if (main_visible) {
		gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(preset_combo), *row + 1);
		gtk_combo_box_text_insert_text(GTK_COMBO_BOX_TEXT(preset_combo), *row, ps->title);
		mom_ps = *row;
		preset_combo_set_item(mom_ps);

		menuitems = gtk_container_get_children(GTK_CONTAINER(tray_menu));
		g_assert(mom_ps < g_list_length(menuitems));
		menuitem = g_list_nth_data(menuitems, mom_ps);
		gtk_widget_destroy(menuitem);
		menuitem = gtk_menu_item_new_with_label(ps->title);

		gtk_menu_shell_insert(GTK_MENU_SHELL(tray_menu), menuitem, *row - 1);
		g_signal_connect(G_OBJECT(menuitem), "activate", (GCallback)preset_menuitem_activate_cb, GINT_TO_POINTER (mom_ps));
		gtk_widget_show(menuitem);
	}

	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list_view), path, NULL, FALSE, 0, 0);
	gtk_tree_path_free(path);

	if (pos + 2 == gtk_tree_model_iter_n_children(GTK_TREE_MODEL(list_store), NULL)) {
		gtk_widget_set_sensitive(move_down_button, FALSE);
	} else {
		gtk_widget_set_sensitive(move_down_button, TRUE);
	}
	gtk_widget_set_sensitive(move_up_button, TRUE);
}

static void destination_button_clicked_cb(GtkWidget *button, gpointer data)
{
	if (rec_settings.destination)
	{
		g_free(rec_settings.destination);
	}
	rec_settings.destination = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(button));
}

static gboolean list_view_key_press_event_cb(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	if (event->keyval == GDK_KEY_Delete)
		remove_button_clicked_cb(widget, user_data);
	if (event->keyval == GDK_KEY_Insert)
		add_button_clicked_cb(widget, user_data);
	
	return FALSE;
}		

static gboolean button_release_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	GtkTreePath *path;
	gint pos;
	GdkEventButton *event_button;
	gchar *buffer;

	event_button = (GdkEventButton *) event;

	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(list_view), event_button->x, event_button->y, &path, NULL, NULL, NULL);
	if (path != NULL) {
		buffer = gtk_tree_path_to_string(path);
		pos = (gint) g_strtod(buffer, NULL);
		g_free(buffer);
		gtk_tree_path_free(path);

		if (pos == 0) {
			gtk_widget_set_sensitive(move_up_button, FALSE);
		} else {
			gtk_widget_set_sensitive(move_up_button, TRUE);
		}

		if (pos + 1 == gtk_tree_model_iter_n_children(GTK_TREE_MODEL(list_store), NULL)) {
			gtk_widget_set_sensitive(move_down_button, FALSE);
		} else {
			gtk_widget_set_sensitive(move_down_button, TRUE);
		}
	}
	return FALSE;
}

static void name_cell_edited_cb(GtkCellRendererText *cellrenderertext, gchar *path_str, gchar *new_val, gpointer user_data)
{
	GtkTreePath *path = NULL;
	GtkTreeIter iter;
	preset *ps;
	int *row;
	GList* menuitems;
	GtkWidget *menuitem;
	
	path = gtk_tree_path_new_from_string(path_str);

	row = gtk_tree_path_get_indices(path);
	g_assert(row);
	g_assert(*row < g_list_length(settings.presets));

	ps = g_list_nth_data(settings.presets, *row);
	g_assert(ps);	
	if (ps->title) g_free(ps->title);
	ps->title = g_strdup(new_val);

	if (main_visible) {
		gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(preset_combo), *row + 1);
		gtk_combo_box_text_insert_text(GTK_COMBO_BOX_TEXT(preset_combo), *row + 1, ps->title);
		mom_ps = *row;
		preset_combo_set_item(mom_ps);

		menuitems = gtk_container_get_children(GTK_CONTAINER(tray_menu));
		g_assert(mom_ps < g_list_length(menuitems));
		menuitem = g_list_nth_data(menuitems, mom_ps);
		gtk_widget_destroy(menuitem);
		menuitem = gtk_menu_item_new_with_label(ps->title); 
			
		gtk_menu_shell_insert(GTK_MENU_SHELL(tray_menu), menuitem, *row);		
		g_signal_connect(G_OBJECT(menuitem), "activate", (GCallback)preset_menuitem_activate_cb, GINT_TO_POINTER (mom_ps));
		gtk_widget_show(menuitem);
	}
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(list_store), &iter, path);
	gtk_list_store_set(GTK_LIST_STORE(list_store), &iter, 0, new_val, -1);
	gtk_tree_path_free(path);	
}	

static void freq_cell_edited_cb(GtkCellRendererText *cellrenderertext, gchar *path_str, gchar *new_val, gpointer user_data)
{
	GtkTreePath *path = NULL;
	GtkTreeIter iter;
	preset *ps;
	int *row;
	double value;
	gchar *freq_str;
	
	if (sscanf(new_val, "%lf", &value) != 1) return;
	
	if (value < FREQ_MIN) value = FREQ_MIN;
	if (value > FREQ_MAX) value = FREQ_MAX;
	value = rint(value * STEPS) / STEPS;
	
	freq_str = g_strdup_printf("%.2f", value);
	
	path = gtk_tree_path_new_from_string(path_str);
	
	row = gtk_tree_path_get_indices(path);
	g_assert(row);
	g_assert(*row < g_list_length(settings.presets));

	ps = g_list_nth_data(settings.presets, *row);
	g_assert(ps);	
	ps->freq = value;

	gtk_adjustment_set_value(adj, value * STEPS);
	mom_ps = *row;
	preset_combo_set_item(mom_ps);
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(list_store), &iter, path);
	gtk_list_store_set(GTK_LIST_STORE(list_store), &iter, 1, freq_str, -1);
	g_free(freq_str);
	gtk_tree_path_free(path);	
}	

static void list_view_cursor_changed_cb(GtkWidget *widget, gpointer data)
{
	int *row;
	preset *ps;
	gint pos;
	GtkTreePath *path = NULL;
	GtkTreeViewColumn *focus_column = NULL;
	
	if (settings.presets == NULL)
		return;

	gtk_tree_view_get_cursor(GTK_TREE_VIEW(list_view), &path, &focus_column);
	
	if (!path)
		return;

	row = gtk_tree_path_get_indices(path);
	g_assert(row);

	mom_ps = *row;
	preset_combo_set_item(mom_ps);

	ps = g_list_nth_data(settings.presets, *row);
	g_assert(ps);

	pos = g_list_index(settings.presets, (gpointer)ps);

	if (pos == 0) {
		gtk_widget_set_sensitive(move_up_button, FALSE);
	} else {
		gtk_widget_set_sensitive(move_up_button, TRUE);
	}

	if (pos + 1 == gtk_tree_model_iter_n_children(GTK_TREE_MODEL(list_store), NULL)) {
			gtk_widget_set_sensitive(move_down_button, FALSE);
	} else {
			gtk_widget_set_sensitive(move_down_button, TRUE);
	}
	gtk_tree_path_free(path);
	return;
}

static void list_view_selection_changed_cb(GtkWidget *widget, gpointer data)
{
	gboolean sel;

	sel = gtk_tree_selection_get_selected(selection, NULL, NULL);

	gtk_widget_set_sensitive(remove_button, sel);
	gtk_widget_set_sensitive(save_button, sel);
}

static void list_view_scroll_to_active_preset_cb(GtkWidget *widget, gpointer data)
{
	GtkTreePath *path = NULL;
	GtkTreeIter iter;
	gint active;

	if (settings.presets == NULL) {
		gtk_widget_grab_focus(add_button);
		return;
	}

	active = gtk_combo_box_get_active(GTK_COMBO_BOX(preset_combo)) - 1;

	if (active < 0) {
		gtk_widget_grab_focus(add_button);
		return;
	}

	if (!gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(list_store), &iter, NULL, active))
		return;

	path = gtk_tree_model_get_path(GTK_TREE_MODEL(list_store), &iter);

	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW (list_view), path, NULL, FALSE, 0, 0);
	gtk_tree_view_set_cursor(GTK_TREE_VIEW (list_view), path, NULL, FALSE );
	gtk_tree_path_free(path);

	gtk_widget_grab_focus(list_view);
}

static void free_presets_list (gpointer data)
{
	preset *ps = (preset *) data;
	g_free (ps->title);
	g_free (ps);
}

/* Go through each row and add its data to the xmlDocPtr */
static gboolean save_to_file_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, xmlNodePtr root, gpointer user_data)
{
	gchar *title, *freq, *position;
	xmlNodePtr current;

	/* get the data stored in the model... */
	gtk_tree_model_get(model, iter, 0, &title, 1, &freq, -1);
	/* ...and get the path of the current row */
	position = gtk_tree_path_to_string(path);
	/* create a new child of the root node... */
	/* (note that I'm using a (guchar*) cast; this is because it's the same thing as
	 * an (xmlChar*) cast, but easier to type) */
	current = xmlNewChild(root, NULL, (guchar*)"station", NULL);
	/* ...and set some properties */
	xmlSetProp(current, (guchar*)"name", (guchar*)title);
	xmlSetProp(current, (guchar*)"freq", (guchar*)freq);
	xmlSetProp(current, (guchar*)"position", (guchar*)position);
	/* free our data we retrieved from the model */
	g_free(title);
	g_free(freq);
	g_free(position);

	/* return FALSE to keep iterating */
	return FALSE;
}

/* Function handle saving an xml file; calls save_to_file_foreach */
static void save_to_file(gchar *filename)
{
	GtkTreeModel *model;
	xmlDocPtr doc;
	xmlNodePtr root;

	/* create a new doc node */
	doc = xmlNewDoc((guchar*)"1.0");
	/* create a new root element. */
	root = xmlNewDocNode(doc, NULL, (guchar*)"gnomeradio", NULL);
	/* set it as the root element */
	xmlDocSetRootElement(doc, root);
	/* get the tree view's model... */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(list_view));
	/* ...and go through it with a foreach */
	gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)save_to_file_foreach, (gpointer)root);
	/* save the actual file */
	xmlSaveFile(filename, doc);
	/* free the doc node */
	xmlFreeDoc(doc);
}

/* Callback for the "Save to file" button; calls save_to_file */
static void save_to_file_cb(GtkWidget *button, gpointer data)
{
	GtkWidget *dialog;
	GtkFileFilter *file_filter_all;
	GtkFileFilter *file_filter_xml;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Select file name\xE2\x80\xA6", NULL,
						GTK_FILE_CHOOSER_ACTION_SAVE,
						GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
						GTK_STOCK_CANCEL,
						GTK_RESPONSE_CANCEL, NULL);

	file_filter_all = gtk_file_filter_new();
	gtk_file_filter_set_name(file_filter_all, _("All Files"));
	gtk_file_filter_add_pattern(file_filter_all, "*");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), file_filter_all);

	file_filter_xml = gtk_file_filter_new();
	gtk_file_filter_set_name(file_filter_xml, _("XML Files (*.xml)"));
	gtk_file_filter_add_pattern(file_filter_xml, "*.xml");
	gtk_file_filter_add_pattern(file_filter_xml, "*.XML");
	gtk_file_filter_add_mime_type(file_filter_xml, "text/xml");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), file_filter_xml);

	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
        gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), file_filter_xml);
	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), "gnomeradio.xml");

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		/* Check if .xml extension is added, if not add */
		if (!g_str_has_suffix(filename, ".xml")) {
			char *tmp_file;
			/* create copy of filename to release it and create a new one, using old name */
			tmp_file = g_strdup(filename);
			g_free(filename);
        		filename = g_strdup_printf("%s.xml", tmp_file);
			g_free(tmp_file);
		}
		save_to_file(filename);
		g_free(filename);
	}
	gtk_widget_destroy(dialog);
}

/* Gets the parent of a path string.
* passing "0:1:2" would return "0:1",
* passing "0:1" would return "0",
* passing "0" would return NULL */
gchar *gtk_tree_path_string_get_parent(gchar *path)
{
	gchar *colon;

	g_return_val_if_fail(path != NULL, NULL);

	colon = g_strrstr(path, ":");
	if (colon == NULL)
		  return NULL;

	return g_strndup(path, colon - path);
}

/* Make sure that path exists within model */
static void gtk_tree_model_generate_path(GtkTreeModel *model, gchar *path)
{
	GtkTreeIter iter, parent;
	gchar *temp;

	while (TRUE) {
		/* if this returns TRUE, then this path exists and we're fine */
		if (gtk_tree_model_get_iter_from_string(model, &iter, path))
			break;

		temp = path;
		path = gtk_tree_path_string_get_parent(path);
		/* if there's no parent, then it's toplevel */
		if (path == NULL) {
			if (GTK_IS_TREE_STORE(model))
				gtk_tree_store_append(GTK_TREE_STORE(model), &parent, NULL);
			else
				gtk_list_store_append(GTK_LIST_STORE(model), &parent);
				gtk_tree_model_generate_path(model, temp);
				break;
		}
		if (GTK_IS_TREE_STORE(model)) {
			gtk_tree_model_generate_path(model, path);
			gtk_tree_model_get_iter_from_string(model, &parent, path);
			gtk_tree_store_append(GTK_TREE_STORE(model), &iter, &parent);
		}
	}
}

/* Function to load from an xml file */
static void load_from_file(gchar *filename)
{
	xmlDocPtr doc;
	xmlNodePtr current;
	xmlChar *title, *freq, *position;
	GtkTreeModel *model;
	GtkTreePath *path = NULL;
	GtkTreeIter iter;
	GtkAdjustment* v_scb;
	GList* menuitems;
	GtkWidget *menuitem;

	/* load the file */
	doc = xmlParseFile(filename);
	/* get the root item */
	if (doc == NULL)
		return;

	current = xmlDocGetRootElement(doc);
	if (current == NULL) {
		xmlFreeDoc(doc);
		return;
	}

	if (xmlStrcmp(current->name, (guchar*)"gnomeradio")) {
		xmlFreeDoc(doc);
		return;
	}

	/* remove previous presets */
	if (settings.presets != NULL) {
		if (main_visible) {
			gint i, count;

			count = g_list_length (settings.presets);
			for (i = 0; i < count; i++)
				gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(preset_combo), 1);

			menuitems = gtk_container_get_children (GTK_CONTAINER (tray_menu));

			count = g_list_length (menuitems);
			for (i = 0; i < count - 6; i++) {
				menuitem = g_list_nth_data (menuitems, i);
				gtk_widget_destroy (menuitem);
			}
       		}

		gtk_list_store_clear(GTK_LIST_STORE(list_store));

		g_list_free_full (settings.presets, free_presets_list);
		settings.presets = NULL;
       	}

	/* get the tree view's model */
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(list_view));

	/* iterate through the root's children items */
	current = current->xmlChildrenNode;
	while (current) {
        	if(!xmlIsBlankNode(current)) {
			/* check for the proper element name */
			if (!xmlStrcmp(current->name, (guchar*)"station")) {
				/* get the saved properties */
				title = xmlGetProp(current, (guchar*)"name");
				freq = xmlGetProp(current, (guchar*)"freq");
				position = xmlGetProp(current, (guchar*)"position");

				if( title && freq && position ) {
					preset *ps = g_malloc0(sizeof(preset));
					ps->title = g_strdup((gchar*)title);
					ps->freq = atof((gchar*)freq);

					settings.presets = g_list_append(settings.presets, ps);

					/* make sure that the path exists */
					gtk_tree_model_generate_path(model, (gchar*)position);
					/* get an iter to the path */
					gtk_tree_model_get_iter_from_string(model, &iter, (gchar*)position);
					/* set the data */
					gtk_list_store_set(GTK_LIST_STORE(list_store), &iter, 0, title, 1, freq, -1);

					gtk_tree_selection_unselect_all(selection);

					v_scb = gtk_scrollable_get_vadjustment(GTK_SCROLLABLE(list_view));
					gtk_adjustment_set_value(v_scb, gtk_adjustment_get_upper(v_scb));

					if (main_visible) {
						gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(preset_combo), ps->title);
						mom_ps = g_list_length(settings.presets) - 1;
						preset_combo_set_item(mom_ps);

						menuitem = gtk_menu_item_new_with_label(ps->title);

						gtk_menu_shell_insert(GTK_MENU_SHELL(tray_menu), menuitem, mom_ps);
						g_signal_connect(G_OBJECT(menuitem), "activate",
								 (GCallback)preset_menuitem_activate_cb,
								 GINT_TO_POINTER (mom_ps));
						gtk_widget_show(menuitem);
					}
				}
				/* free the data */
				xmlFree(title);
				xmlFree(freq);
				xmlFree(position);
			}
			current = current->next;
		}
	}
	/* free the doc node */
	xmlFreeDoc(doc);

	if (settings.presets == NULL)
		return;

	path = gtk_tree_model_get_path(GTK_TREE_MODEL(list_store), &iter);

	gtk_tree_view_set_cursor(GTK_TREE_VIEW (list_view), path, NULL, FALSE );
	gtk_tree_path_free(path);

	gtk_widget_grab_focus(list_view);

	list_view_cursor_changed_cb(list_view, NULL);
	gtk_widget_set_sensitive(save_button, TRUE);
	gtk_widget_set_sensitive(remove_button, TRUE);
}

/* Callback for the "Load from file" button; calls load_from_file */
static void load_from_file_cb(GtkWidget *button, gpointer data)
{
	GtkWidget *dialog;
	GtkFileFilter *file_filter_all;
	GtkFileFilter *file_filter_xml;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Select file name\xE2\x80\xA6", NULL,
						GTK_FILE_CHOOSER_ACTION_OPEN,
						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
						GTK_STOCK_CANCEL,
						GTK_RESPONSE_CANCEL, NULL);

	file_filter_all = gtk_file_filter_new();
	gtk_file_filter_set_name(file_filter_all, _("All Files"));
	gtk_file_filter_add_pattern(file_filter_all, "*");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), file_filter_all);

	file_filter_xml = gtk_file_filter_new();
	gtk_file_filter_set_name(file_filter_xml, _("XML Files"));
	gtk_file_filter_add_pattern(file_filter_xml, "*.xml");
	gtk_file_filter_add_pattern(file_filter_xml, "*.XML");
	gtk_file_filter_add_mime_type(file_filter_xml, "text/xml");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), file_filter_xml);

        gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), file_filter_xml);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		load_from_file(filename);
		g_free(filename);
	}
	gtk_widget_destroy(dialog);
}

gboolean action_mode (gint mode)
{
	switch (mode) {
		case RADIO_DEVICE:
			gtk_widget_grab_focus (device_entry);
			break;
		case MIXER_DEVICE:
			gtk_widget_grab_focus (mixer_entry);
			break;
		case PRESETS:
			list_view_scroll_to_active_preset_cb(list_view, NULL);
			break;
		default:
			break;
	}

	return TRUE;
}

GtkWidget* prefs_window(GtkWidget *app)
{
	GtkWidget *dialog;
	GtkWidget *box;
	GtkWidget *grid;
	GtkWidget *settings_label, *presets_label, *record_label;
	GtkWidget *s_indent_label, *p_indent_label, *r_indent_label;
	GtkWidget *destination_label;
	GtkWidget *destination_button;
	GtkWidget *audio_profile_label;
	GtkWidget *device_label, *mixer_label;
	GtkWidget *button_box;
	GtkWidget *open_button;
	GtkWidget *scrolled_window;
	GtkCellRenderer *cellrenderer;
	GtkTreeViewColumn *list_column;

	GstEncodingProfile *profile;
	gint i;
	char *settings_hdr, *presets_hdr, *record_hdr;
	preset* ps;
	GList *l;
	
	dialog = gtk_dialog_new_with_buttons(_("Gnomeradio Settings"), GTK_WINDOW(app), 
			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, 
			GTK_STOCK_HELP, GTK_RESPONSE_HELP,
			NULL);
	
	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CLOSE);

	box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 15);
	gtk_container_set_border_width(GTK_CONTAINER(box), 10);
	gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), box, TRUE, TRUE, 0);
	
	grid = gtk_grid_new();
	gtk_grid_set_row_spacing(GTK_GRID(grid), 5);
	gtk_grid_set_column_spacing(GTK_GRID(grid), 20);
	gtk_box_pack_start(GTK_BOX(box), grid, TRUE, TRUE, 0);

	/* The general settings part */
	settings_hdr = g_strconcat("<span weight=\"bold\">", _("General Settings"), "</span>", NULL);
	settings_label = gtk_label_new(settings_hdr);
	gtk_widget_set_halign(settings_label, GTK_ALIGN_START);
	gtk_label_set_use_markup(GTK_LABEL(settings_label), TRUE);
	g_free(settings_hdr);
	gtk_grid_attach(GTK_GRID(grid), settings_label, 0, 0, 2, 1);

	s_indent_label = gtk_label_new("");
	gtk_grid_attach(GTK_GRID(grid), s_indent_label, 0, 1, 1, 3);

	device_label = gtk_label_new(_("Radio Device:"));
	gtk_widget_set_halign(device_label, GTK_ALIGN_START);
	device_entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(device_entry), settings.device);
	gtk_grid_attach(GTK_GRID(grid), device_label, 1, 1, 1, 1);
	gtk_grid_attach(GTK_GRID(grid), device_entry, 2, 1, 1, 1);

	mixer_label = gtk_label_new(_("Mixer Source:"));
	gtk_widget_set_halign(mixer_label, GTK_ALIGN_START);
	mixer_entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(mixer_entry), settings.mixer);
	
	gtk_grid_attach(GTK_GRID(grid), mixer_label, 1, 2, 1, 1);
	gtk_grid_attach(GTK_GRID(grid), mixer_entry, 2, 2, 1, 1);

	mute_on_exit_cb = gtk_check_button_new_with_label(_("Mute on exit"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mute_on_exit_cb), settings.mute_on_exit);

	gtk_grid_attach(GTK_GRID(grid), mute_on_exit_cb, 1, 3, 1, 1);

	g_signal_connect(G_OBJECT(device_entry), "hide", G_CALLBACK(device_entry_activate_cb), app);
	g_signal_connect(G_OBJECT(device_entry), "activate", G_CALLBACK(device_entry_activate_cb), NULL);
	g_signal_connect(G_OBJECT(mixer_entry), "hide", G_CALLBACK(mixer_entry_activate_cb), app);
	g_signal_connect(G_OBJECT(mixer_entry), "activate", G_CALLBACK(mixer_entry_activate_cb), NULL);
	g_signal_connect(G_OBJECT(mute_on_exit_cb), "toggled", G_CALLBACK(mute_on_exit_toggled_cb), NULL);

	gtk_widget_set_tooltip_text(device_entry, _("The radio device to use (e.g. /dev/radio0)"));
	gtk_widget_set_tooltip_text(mixer_entry, _("The mixer device and channel to use (e.g. hw:0/Line)"));
	gtk_widget_set_tooltip_text(mute_on_exit_cb, _("Mute radio device on exit"));

	/* The presets part */
	presets_hdr = g_strconcat("<span weight=\"bold\">", _("Presets"), "</span>", NULL);
	presets_label = gtk_label_new(presets_hdr);
	gtk_widget_set_halign(presets_label, GTK_ALIGN_START);
	gtk_label_set_use_markup(GTK_LABEL(presets_label), TRUE);
	g_free(presets_hdr);
	gtk_grid_attach(GTK_GRID(grid), presets_label, 0, 4, 2, 1);

	p_indent_label = gtk_label_new("");
	gtk_grid_attach(GTK_GRID(grid), p_indent_label, 0, 5, 1, 2);

	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(scrolled_window), 75);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window), GTK_SHADOW_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	list_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
	list_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(list_store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW(list_view), TRUE);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(list_view), FALSE);

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list_view));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
	gtk_container_add(GTK_CONTAINER(scrolled_window), list_view);

	g_signal_connect(G_OBJECT(list_view), "button-release-event", G_CALLBACK(button_release_cb), NULL);
	g_signal_connect(G_OBJECT(list_view), "key-press-event", G_CALLBACK(list_view_key_press_event_cb), NULL);
	g_signal_connect(G_OBJECT(list_view), "cursor-changed", G_CALLBACK(list_view_cursor_changed_cb), NULL);

	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(list_view_selection_changed_cb), list_view);

	/*gtk_tree_selection_select_path(selection, gtk_tree_path_new_from_string("0"));
	gtk_tree_selection_set_select_function(selection, (GtkTreeSelectionFunc)list_view_select_cb, NULL, NULL);*/
	
	cellrenderer = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(cellrenderer), "mode", GTK_CELL_RENDERER_MODE_EDITABLE, NULL);
	g_object_set(G_OBJECT(GTK_CELL_RENDERER_TEXT(cellrenderer)), "editable", TRUE, NULL);
	list_column = gtk_tree_view_column_new_with_attributes(NULL, cellrenderer, "text", 0, NULL);
	gtk_tree_view_column_set_reorderable(list_column, TRUE);
	gtk_tree_view_column_set_expand(list_column, TRUE);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), list_column);

	g_signal_connect(G_OBJECT(cellrenderer), "edited", G_CALLBACK(name_cell_edited_cb), NULL);

	cellrenderer = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(cellrenderer), "mode", GTK_CELL_RENDERER_MODE_EDITABLE, NULL);
	g_object_set(G_OBJECT(cellrenderer), "xalign", 1.0, NULL);
	g_object_set(G_OBJECT(GTK_CELL_RENDERER_TEXT(cellrenderer)), "editable", TRUE, NULL);
	list_column = gtk_tree_view_column_new_with_attributes(NULL, cellrenderer, "text", 1, NULL);
	gtk_tree_view_column_set_reorderable(list_column, TRUE);
	gtk_tree_view_column_set_expand(list_column, FALSE);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), list_column);

	g_signal_connect(G_OBJECT(cellrenderer), "edited", G_CALLBACK(freq_cell_edited_cb), NULL);

	i = 0;
	for (l = settings.presets; l != NULL; l = l->next) {
		GtkTreeIter iter = {0};
		char *buffer;
		ps = l->data;
		buffer = g_strdup_printf("%0.2f", ps->freq);
		gtk_list_store_append(list_store, &iter);
		gtk_list_store_set(GTK_LIST_STORE(list_store), &iter, 0, ps->title, 1, buffer, -1);
		g_free(buffer);
		i++;
	}

	gtk_grid_attach(GTK_GRID(grid), scrolled_window, 1, 5, 2, 2);

	button_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);

	move_up_button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(move_up_button), gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU));
	gtk_button_set_relief (GTK_BUTTON (move_up_button), GTK_RELIEF_NONE);
	gtk_widget_set_sensitive(move_up_button, FALSE);
	gtk_widget_set_tooltip_text(move_up_button, _("Move preset up"));

	g_signal_connect(G_OBJECT(move_up_button), "clicked", G_CALLBACK(move_up_button_clicked_cb), NULL);

	move_down_button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(move_down_button), gtk_image_new_from_stock(GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU));
	gtk_button_set_relief (GTK_BUTTON (move_down_button), GTK_RELIEF_NONE);
	gtk_widget_set_sensitive(move_down_button, FALSE);
	gtk_widget_set_tooltip_text(move_down_button, _("Move preset down"));

	g_signal_connect(G_OBJECT(move_down_button), "clicked", G_CALLBACK(move_down_button_clicked_cb), NULL);

	add_button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(add_button), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
	gtk_button_set_relief (GTK_BUTTON (add_button), GTK_RELIEF_NONE);
	gtk_widget_set_tooltip_text(add_button, _("Add preset"));

	g_signal_connect(G_OBJECT(add_button), "clicked", G_CALLBACK(add_button_clicked_cb), NULL);

 	remove_button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(remove_button), gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU));
	gtk_button_set_relief (GTK_BUTTON (remove_button), GTK_RELIEF_NONE);
	gtk_widget_set_tooltip_text(remove_button, _("Remove preset"));
 	gtk_widget_set_sensitive(remove_button, FALSE);

	g_signal_connect(G_OBJECT(remove_button), "clicked", G_CALLBACK(remove_button_clicked_cb), NULL);

 	save_button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(save_button), gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_MENU));
	gtk_button_set_relief (GTK_BUTTON (save_button), GTK_RELIEF_NONE);
	gtk_widget_set_tooltip_text(save_button, _("Save to file\xE2\x80\xA6"));

	if (settings.presets == NULL) {
		gtk_widget_set_sensitive(save_button, FALSE);
	} else {
		gtk_widget_set_sensitive(save_button, TRUE);
	}

	g_signal_connect(G_OBJECT(save_button), "clicked", G_CALLBACK(save_to_file_cb), NULL);

 	open_button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(open_button), gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU));
	gtk_button_set_relief (GTK_BUTTON (open_button), GTK_RELIEF_NONE);
	gtk_widget_set_tooltip_text(open_button, _("Load from file\xE2\x80\xA6"));
	gtk_widget_set_sensitive(open_button, TRUE);

	g_signal_connect(G_OBJECT(open_button), "clicked", G_CALLBACK(load_from_file_cb), NULL);

	gtk_box_pack_end(GTK_BOX(button_box), move_down_button, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(button_box), move_up_button, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(button_box), remove_button, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(button_box), add_button, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(button_box), save_button, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(button_box), open_button, FALSE, FALSE, 0);

	gtk_grid_attach(GTK_GRID(grid), button_box, 2, 7, 1, 1);

	/* The record settings part */
	record_hdr = g_strconcat("<span weight=\"bold\">", _("Record Settings"), "</span>", NULL);
	record_label = gtk_label_new(record_hdr);
	gtk_widget_set_halign(record_label, GTK_ALIGN_START);
	gtk_label_set_use_markup(GTK_LABEL(record_label), TRUE);
	g_free(record_hdr);
	gtk_grid_attach(GTK_GRID(grid), record_label, 0, 8, 2, 1);

	r_indent_label = gtk_label_new("");
	gtk_grid_attach(GTK_GRID(grid), r_indent_label, 0, 9, 1, 2);

	destination_label = gtk_label_new(_("Destination:"));
	gtk_widget_set_halign(destination_label, GTK_ALIGN_START);

	destination_button = gtk_file_chooser_button_new(
		_("Select a folder"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(destination_button), rec_settings.destination);
	
	audio_profile_label = gtk_label_new(_("Format:"));
	gtk_widget_set_halign(audio_profile_label, GTK_ALIGN_START);
	audio_profile_combo = audio_profile_choose_new();
	audio_profile_chooser_set_active(audio_profile_combo, rec_settings.profile);

	install_plugins_button = gtk_button_new_with_label(_("Install additional software required to use this format"));
	gtk_widget_set_no_show_all (install_plugins_button, TRUE);

	profile = rb_gst_get_encoding_profile (rec_settings.profile);
	if (rb_gst_check_missing_plugins (profile, NULL, NULL)) {
		gtk_widget_set_visible (install_plugins_button, TRUE);
		gtk_widget_set_sensitive (install_plugins_button,
					gst_install_plugins_supported ());
	} else {
		gtk_widget_set_visible (install_plugins_button, FALSE);
	}

	gtk_grid_attach(GTK_GRID(grid), destination_label, 1, 9, 1, 1);
	gtk_grid_attach(GTK_GRID(grid), destination_button, 2, 9, 1, 1);
	gtk_grid_attach(GTK_GRID(grid), audio_profile_label, 1, 10, 1, 1);
	gtk_grid_attach(GTK_GRID(grid), audio_profile_combo, 2, 10, 1, 1);
	gtk_grid_attach(GTK_GRID(grid), install_plugins_button, 2, 11, 1, 1);

	g_signal_connect(GTK_FILE_CHOOSER(destination_button), "selection-changed", G_CALLBACK(destination_button_clicked_cb), NULL);
	g_signal_connect(G_OBJECT(audio_profile_combo), "changed", G_CALLBACK(audio_profile_combo_change_cb), NULL);
	g_signal_connect(G_OBJECT(install_plugins_button), "clicked", G_CALLBACK(audio_profile_install_plugins_cb), NULL);

	gtk_widget_set_tooltip_text(destination_button, _("Select a location where to save recorded file."));
	gtk_widget_set_tooltip_text(audio_profile_combo, _("Choose the format that should be used for recording."));

	gtk_widget_show_all(dialog);

	return dialog;
}
